/*global $, _, Qv, NProgress*/
/*jshint strict: false*/
/**
 * Property Mapping:
 *
 *     Chart.Expression.0.0 id_field
 *     Layout.Text0 api_token
 *     Layout.Text1 project_id
 *     Layout.Text2 cluster_url
 *     Layout.Text3 verified URL (hidden from user)
 *     Layout.Text4 id_field_name
 *     Layout.Text5 noise_level
 **/

function SquirroDropzone_Widget(qv) {
    this.qv = qv;
    this.$qv = $(qv.Element);
    this.render(qv);
    this.files = [];
    this.xhrs = [];

    // Clear current selection
    this.qv.Data.SelectTextsInColumn(0, false, []);

    var that = this,
        doc = Qv.GetDocument();

    doc.GetCurrentSelections({
        onChange: function () {
            if (_.isEmpty(this.Data.Rows)) {
                NProgress.done();
                that._clear();
            }
        },
        fields: [{
            name: this.qv.Data.HeaderRows[0][0].text
        }]
    });
}


/**
 * Returns the extension's configuration.
 */
 SquirroDropzone_Widget.prototype._getParams = function () {
    return {
        api_token: this._getValue(0),
        project_id: this._getValue(1),
        cluster_url: this._getValue(2),
        verified_url: this._getValue(3),
        id_field_name: this._getValue(4),
        noise_level: parseFloat(this._getValue(5)) || 0
    };
};

/**
 * Returns configuration options of a given index.
 */
 SquirroDropzone_Widget.prototype._getValue = function (idx) {
    var data = this.qv.Layout['Text' + idx],
        retval = null;
    if (data && data.text && data.text !== '-') {
        retval = $.trim(data.text);
    }
    return retval;
};


SquirroDropzone_Widget.prototype.render = function (qv) {
    if (qv && this.qv !== qv) {
        // Re-render in the new QV context
        this.el = null;
        this.qv = qv;
        this.$qv = $(qv.Element);
    }

    if (! this.el) {
        this._render();
        this._attachEvents();
    }

    var params = this._getParams();
    if (params.noise_level !== this._noiseLevel && this._fingerprint) {
        // Update search with new noise level
        this._searchForFingerprint();
    }
    this._noiseLevel = params.noise_level;
    if (!this.url) {
        this.url = params.verified_url;
    }
    if (this.url !== params.cluster_url) {
        // Server URL has changed, validate it
        this._validateUrl(params.cluster_url);
    }
};

SquirroDropzone_Widget.prototype._validateUrl = function (url) {
    var that = this;

    $.ajax({
        url: url + '/app/bulk/iframe_isalive',
        type: 'GET',
        dataType: 'jsonp',
        jsonp: false, // Do not add ?callback to the URL
        jsonpCallback: 'jsonp',
        timeout: 5000
    }).done(function () {
        // Server is alive, set verified URL
        that.url = url;
        that.qv.Layout.Text3.text = url;
        that.qv.Layout.SetProperty('Chart.Text.3.Content', url);
    }).fail(function () {
        that.el.find('.error').text('Squirro server not reachable').show();
    });
};

SquirroDropzone_Widget.prototype._render = function () {

    var caption = this.qv.Layout.Caption.visible ? ' caption' : '',
        html = [
            '<div class="dropzone' + caption + '">',
            '<div class="msg"></div>',
            '<ul class="files"></ul>',
            '</div>'
        ].join('');
    this.el = $(html);
    this.el.click(_.bind(this._removeFiles, this));
    this.$qv.html(this.el);


    _.each(this.files, function (file) {
        this._appendFile(file);
    }, this);
};

SquirroDropzone_Widget.prototype._msg = function (message) {
    this.el.find('.msg')
        .removeClass('hide')
        .html(message);
};

SquirroDropzone_Widget.prototype._clear = function () {
    _.invoke(this.xhrs, 'abort');
    this.xhrs = [];
    this.files = [];
    this.el.find('.files').empty();
};

SquirroDropzone_Widget.prototype._removeFiles = function () {
    this.qv.Data.SelectTextsInColumn(0, false, []);
    this._clear();
};

SquirroDropzone_Widget.prototype._appendFile = function (file) {
    this.el.find('.msg').addClass('hide');
    this.el.find('.files').append('<li class="QvSelected js-file">' + file + '</li>');
};

SquirroDropzone_Widget.prototype._attachEvents = function () {
    var that = this,
        params = this._getParams(),
        uploadedFiles = {};

    this.el.fileReaderJS({
        readAsDefault: 'DataURL',
        on: {
            groupstart: function () {
                that._clear();
                uploadedFiles = {};
                NProgress.configure({
                    parent: that.el,
                    showSpinner: false
                });
                NProgress.start();
            },
            loadend: function (e, file) {
                uploadedFiles[file.name] = e.target.result;
                NProgress.inc();
            },
            groupend: function(list) {
                var entries = _.filter(_.map(list.files, function (file) {
                    var result = uploadedFiles[file.name];
                    that.files.push(file.name);
                    that._msg('Uploading\u2026');
                    delete uploadedFiles[file.name];
                    if (result){
                        return {
                            data: result,
                            mime: file.type,
                            filename: file.name
                        };
                    }
                    return null;
                }), _.isObject);

                var url = [
                    that.url,
                    '/v0/fingerprint/project/',
                    params.project_id,
                    '?token=',
                    params.api_token
                ].join('');
                var xhr = $.ajax({
                    url:  url,
                    type: 'POST',
                    contentType: 'application/json',
                    dataType: 'json',
                    data: JSON.stringify({
                        entries: entries
                    })
                });

                xhr.done(function (response) {
                    that._fingerprint = response.name;
                    that._searchForFingerprint();
                });
                xhr.fail(function () {
                    that.el.addClass('error');
                    that._msg('Upload failed.');
                });
                that.xhrs.push(xhr);
            }
        }
    });
};

SquirroDropzone_Widget.prototype._searchForFingerprint = function () {
    var params = this._getParams(),
        that = this;

    var xhr = $.ajax({
        url: this.url + '/v0/items',
        dataType: 'jsonp',
        data: {
            query: 'smartfilter:' + this._fingerprint + ':' + params.noise_level,
            token: params.api_token,
            sort: 'score',
            count: 100,
            project_id: params.project_id
        }
    } );

    xhr.done(function (data) {
        var ids = [],
            i,
            items = data.items;
        for (i = 0; i < items.length; i+=1) {
            // Selection can be made using the Squirro ID, the external ID or
            // any keyword which is present in the item response
            if (params.id_field_name.indexOf('keywords.') > -1) {
                var id_field_name = params.id_field_name.split('.')[1];
                if (items[i]['keywords'][id_field_name]) {
                    ids.push(items[i]['keywords'][id_field_name]);
                }
            } else {
                if (items[i][params.id_field_name]) {
                    ids.push(items[i][params.id_field_name]);
                }
            }
        }
        if (ids.length > 0) {
            ids = _.uniq(_.flatten(ids));
            that.qv.Data.SelectTextsInColumn(0, false, ids);
        }
        that.el.find('.files').empty();
        _.each(that.files, function (file) {
            that._appendFile(file);
        });
        NProgress.done();
    });

    xhr.fail(function () {
        that._clear();
        that.el.addClass('error');
        that._msg('Upload failed.');
    });

    this.xhrs.push(xhr);
};
